@@ -3,10 +3,13 @@ from django.template import Context, RequestContext
 from django.template.loader import get_template
 from django.utils.log import getLogger
 
+import sys
+import six
+import traceback
 from inspect import isclass
 from importlib import import_module
 
-from achilles.common import BaseLibrary, achilles_data
+from achilles.common import BaseLibrary
 from achilles.actions import Library as ActionsLibrary
 
 logger = getLogger(__name__)
@@ -84,7 +87,8 @@ class Library(BaseLibrary):
                     res = func(self.context, *args, **kwargs)
                 else:
                     res = func(*args, **kwargs)
-                context.update(res)
+                if res != context:
+                    context.update(res)
                 return context
         return B
 
@@ -99,10 +103,18 @@ def get(name, context=None):
     """
     # make sure all blocks are loaded
     for app in settings.INSTALLED_APPS:
-        try:
-            import_module(app + '.blocks')
-        except ImportError:
-            pass
+        if six.PY2:
+            try:
+                import_module(app + '.blocks')
+            except ImportError:
+                tb = sys.exc_info()[2]
+                stack = traceback.extract_tb(tb, 3)
+                if len(stack) > 2:
+                    raise
+        else:
+            from importlib import find_loader
+            if find_loader(app + '.blocks'):
+                import_module(app + '.blocks')
 
     return Library.get_global(name)(context)
 
@@ -137,12 +149,11 @@ class Block(object):
         """
         return self.context
 
-    def update(self, request, *args, **kwargs):
+    def update(self, transport, *args, **kwargs):
         """
-        Render and send the update of block within the given request
+        Render and send the update of block within the given achilles transport
         """
-        blocks = achilles_data(request, 'blocks', [])
-        blocks.append({
+        transport.data('blocks', []).append({
             'name': self.register_name,
             'args': args,
             'kwargs': kwargs,
@@ -154,7 +165,7 @@ register = ActionsLibrary('blocks')
 
 
 @register.action
-def update(request, name, *args, **kwargs):
+def update(transport, name, *args, **kwargs):
     """
     Action name: **blocks:update**
 
@@ -165,13 +176,13 @@ def update(request, name, *args, **kwargs):
     the block handler. When using arguments, only blocks matching them
     will be updated.
 
-    :param request: Django request object that is being served
+    :param transport: Achilles transport object that is being served
     :param name: Fully namespaced block name
     """
-    context = RequestContext(request, {})
+    context = RequestContext(transport.request, {})
     block = get(name, context)
-    block.update(request, *args, **kwargs)
+    block.update(transport, *args, **kwargs)
 
 
-def render(request):
-    return achilles_data(request, 'blocks', [])
+def render(transport):
+    return transport.data('blocks', [])
